/*
 ** yafhooks.h
 ** YAF Active Flow Table Plugin Interface
 **
 ** ------------------------------------------------------------------------
 ** Copyright (C) 2007 Carnegie Mellon University. All Rights Reserved.
 ** ------------------------------------------------------------------------
 ** Authors: Brian Trammell <bht@cert.org>
 ** ------------------------------------------------------------------------
 ** @OPENSOURCE_HEADER_START@
 ** Use of the YAF system and related source code is subject to the terms 
 ** of the following licenses:
 ** 
 ** GNU Public License (GPL) Rights pursuant to Version 2, June 1991
 ** Government Purpose License Rights (GPLR) pursuant to DFARS 252.227.7013
 ** 
 ** NO WARRANTY
 ** 
 ** ANY INFORMATION, MATERIALS, SERVICES, INTELLECTUAL PROPERTY OR OTHER 
 ** PROPERTY OR RIGHTS GRANTED OR PROVIDED BY CARNEGIE MELLON UNIVERSITY 
 ** PURSUANT TO THIS LICENSE (HEREINAFTER THE "DELIVERABLES") ARE ON AN 
 ** "AS-IS" BASIS. CARNEGIE MELLON UNIVERSITY MAKES NO WARRANTIES OF ANY 
 ** KIND, EITHER EXPRESS OR IMPLIED AS TO ANY MATTER INCLUDING, BUT NOT 
 ** LIMITED TO, WARRANTY OF FITNESS FOR A PARTICULAR PURPOSE, 
 ** MERCHANTABILITY, INFORMATIONAL CONTENT, NONINFRINGEMENT, OR ERROR-FREE 
 ** OPERATION. CARNEGIE MELLON UNIVERSITY SHALL NOT BE LIABLE FOR INDIRECT, 
 ** SPECIAL OR CONSEQUENTIAL DAMAGES, SUCH AS LOSS OF PROFITS OR INABILITY 
 ** TO USE SAID INTELLECTUAL PROPERTY, UNDER THIS LICENSE, REGARDLESS OF 
 ** WHETHER SUCH PARTY WAS AWARE OF THE POSSIBILITY OF SUCH DAMAGES. 
 ** LICENSEE AGREES THAT IT WILL NOT MAKE ANY WARRANTY ON BEHALF OF 
 ** CARNEGIE MELLON UNIVERSITY, EXPRESS OR IMPLIED, TO ANY PERSON 
 ** CONCERNING THE APPLICATION OF OR THE RESULTS TO BE OBTAINED WITH THE 
 ** DELIVERABLES UNDER THIS LICENSE.
 ** 
 ** Licensee hereby agrees to defend, indemnify, and hold harmless Carnegie 
 ** Mellon University, its trustees, officers, employees, and agents from 
 ** all claims or demands made against them (and any related losses, 
 ** expenses, or attorney's fees) arising out of, or relating to Licensee's 
 ** and/or its sub licensees' negligent use or willful misuse of or 
 ** negligent conduct or willful misconduct regarding the Software, 
 ** facilities, or other rights or assistance granted by Carnegie Mellon 
 ** University under this License, including, but not limited to, any 
 ** claims of product liability, personal injury, death, damage to 
 ** property, or violation of any laws or regulations.
 ** 
 ** Carnegie Mellon University Software Engineering Institute authored 
 ** documents are sponsored by the U.S. Department of Defense under 
 ** Contract F19628-00-C-0003. Carnegie Mellon University retains 
 ** copyrights in all material produced under this contract. The U.S. 
 ** Government retains a non-exclusive, royalty-free license to publish or 
 ** reproduce these documents, or allow others to do so, for U.S. 
 ** Government purposes only pursuant to the copyright license under the 
 ** contract clause at 252.227.7013.
 ** 
 ** @OPENSOURCE_HEADER_END@
 ** ------------------------------------------------------------------------
 */

/**
 * @file
 *
 * Processing hook interface for YAF. Not yet documented.
 */

/*

Design notes:

1. For now, it is okay for the yfhook facility to only support a single hook.

5. Each hook needs to know when a flow is flushed, so that it can make the
per-flow export fields available.

6. Each hook needs to be able to hand YAF an export template for its fields.
These fields will appear in every exported record; a facility for NULLs MUST
be provided by the hook's representation.

*/

#ifndef _YAF_HOOKS_H_
#define _YAF_HOOKS_H_

#include <yaf/autoinc.h>
#include <yaf/decode.h>
#include <yaf/yafcore.h>
#include <yaf/yaftab.h>
#if YAF_ENABLE_APPLABEL
#include <pcre.h>
#endif


#define YAF_HOOK_INTERFACE_VERSION 2

/** Exported from the plugin to tell YAF about its export data & interface version */
struct yfHookMetaData {
    /** version of plugin interface */
    uint8_t version;
    /** size of data plugin will export */
    uint32_t exportDataSize;
    /** turn on application labeling related functions */
    uint8_t requireAppLabel;
};


/**
 * Function called to do processing on each packet as it comes in
 *
 */
gboolean            yfHookPacket (
    yfFlowKey_t * key,
    const uint8_t * pkt,
    size_t caplen,
    uint16_t iplen,
    yfTCPInfo_t * tcpinfo,
    yfL2Info_t * l2info);

/**
 * Similar to yfHookPacket but also given yfFlowVal_t struct for
 * processing per flow direction
 */
void                yfHookFlowPacket (
    yfFlow_t * flow,
    yfFlowVal_t * val,
    const uint8_t *pkt,
    size_t caplen,
    uint16_t iplen,
    yfTCPInfo_t * tcpinfo,
    yfL2Info_t * l2info);
/**
 *
 * Validation function to make sure the plugin can and should operate
 * based on the flowtable options
 *
 * @param    max payload value
 * @param    uniflow
 * @param    silkmode
 * @param    applabelmode
 * @param    p0f finger printing mode
 * @param    handshake header export mode
 * @param    udp max payload - concatenate udp payloads similar to TCP
 * @param    udp uniflow port - export all udp packets if have this src or dst port 
 */
void                yfHookValidateFlowTab (
    uint32_t        max_payload,
    gboolean        uniflow,
    gboolean        silkmode,
    gboolean        applabelmode,
    gboolean        entropymode,
    gboolean        fingerprintmode,
    gboolean        fpExportMode,
    gboolean        udp_max_payload,
    uint16_t        udp_uniflow_port);

/**
 *
 * Called upon flow close to do any necessary 
 * plugin processing upon flow close
 *
 */
gboolean            yfHookFlowClose (
    yfFlow_t * flow);

/**
 *
 * Allow plugins to allocate flow state information for 
 * each flow captured by yaf.
 *
 * @param flow the pointer to the flow context state structure, but
 * more importantly contains the array of pointers (hfctx) which
 * hold the plugin context state
 *
 */
void                yfHookFlowAlloc (
    yfFlow_t * flow);

/**
 * Frees all memory associated with the flow state in all of the
 * attached plugins
 *
 * @param  flow - a pointer to the flow context structure
 *
 */
void                yfHookFlowFree (
    yfFlow_t * flow);

/**
 * returns the IPFIX info model aggregated for all plugins
 *
 * @return pointer to an arry of fbInfoElement_t that contains
 * the sum of the IPFIX IE's from all active plugins
 */
fbInfoElement_t    *yfHookGetInfoModel (
    void);

/**
 *  gets the IPFIX info model template for the export data from _all_
 *  the plugins and turns it into a single template to return.  It caches
 *  the results so that future queries are a lot faster.  It can validate
 *  the cached result if the numer of plugins registered changes.
 * 
 *  @param pointer to an array of fbInfoElementSpec_t structures that describes
 *  the info model template
 */
fbInfoElementSpec_t *yfHookGetTemplate (
    void);

/**
 * called by yfWriteFlow to add the data from all registered plugins
 * to the outgoing IPFIX record
 *
 * @param   outgoing yfIpfixFlow_t record
 * @param   size of record
 * @param   pointer to the flow context structure
 * @param   Error
 */
gboolean            yfWriteFlowHook (
    void *rec,
    size_t rec_sz,
    yfFlow_t * flow,
    GError ** err);

/**
 * yfHookAddNewHook
 *
 * adds another hook (plugin) into yaf
 *
 * \param hookName the file name of the plugin to load
 * \param hookOpts a string of command line options for the plugin to process
 * \param err the error value that gets set if this call didn't work
 *
 * \return TRUE if plugin loaded fine, other FALSE
 *
 */

gboolean            yfHookAddNewHook (
    const char *hookName,
    const char *hookOpts,
    GError ** err);


#if YAF_ENABLE_APPLABEL
/**
 * Specifically if application labeling is enabled for deep packet
 * inspection plugin.
 */
void yfHookScanPayload (
	yfFlow_t *flow,
	const uint8_t *pkt,
	size_t caplen,
	pcre *expression,
	uint16_t offset,
	uint16_t elementID,
	uint16_t applabel);

#endif


#endif
